@@ -1,6 +1,7 @@
 from albatross import status_codes
-import ujson as json
+from albatross.compat import json
 from albatross.data_types import CaselessDict
+from albatross.http_error import HTTPError
 
 
 class Response:
@@ -11,12 +12,13 @@ class Response:
         body (str):
 
     """
+
     def __init__(self):
         self.status_code = status_codes.HTTP_200
         self._chunks = []
-        self.headers = CaselessDict(
-           ('Content-Type', 'text/html')
-        )
+        self.headers = CaselessDict([
+            ('Content-Type', 'text/html')
+        ])
         self.cookies = {}
 
     def clear(self):
@@ -38,3 +40,4 @@ class Response:
             self.status_code = status_codes.HTTP_301
         else:
             self.status_code = status_codes.HTTP_302
+        raise HTTPError(self.status_code)
