@@ -1,14 +1,31 @@
 import re
 import asyncio
-import urllib.parse as parse
 from datetime import datetime
 from albatross import Request, Response
 from albatross.status_codes import HTTP_404, HTTP_500, HTTP_405
 from albatross.http_error import HTTPError
-from albatross.data_types import ImmutableCaselessDict
+from httptools import HttpRequestParser
 import traceback
 
 
+def write_cookie(writer, key, value):
+    if isinstance(value, tuple):
+        value, duration = value
+        if isinstance(duration, datetime):
+            format = duration.strftime('%a %d %b %Y %H:%M:%S GMT').encode()
+            writer.write(b'Set-Cookie: %s=%s;expires=%s\r\n' % (
+                key.encode(), str(value).encode(), format)
+            )
+        elif isinstance(duration, int):
+            writer.write(b'Set-Cookie: %s=%s;max-age=%d\r\n' % (
+                key.encode(), str(value).encode(), duration)
+            )
+    else:
+        writer.write(b'Set-Cookie: %s=%s\r\n' % (
+            key.encode(), str(value).encode())
+        )
+
+
 class Server:
     """The core albatross server
 
@@ -29,59 +46,36 @@ class Server:
                 return handler, match.groupdict()
         return None, None
 
-    def add_route(self, route, handler):
+    def add_regex_route(self, route, handler):
         route += '$'
         compiled = re.compile(route)
         self._handlers.append((compiled, handler))
 
+    def add_route(self, route, handler):
+        route = re.sub('{([-_a-zA-Z]+)}', '(?P<\g<1>>[^/?]+)', route)
+        self.add_regex_route(route, handler)
+
     def add_middleware(self, middleware):
         self._middleware.append(middleware)
 
-    def _parse_header_lines(self, lines):
-        headers = {}
-        for l in lines:
-            try:
-                key, value = l.split(':', 1)
-                headers[key] = value.strip()
-            except ValueError:
-                pass
-        return ImmutableCaselessDict(*headers.items())
+    async def _parse_request(self, request_reader, response_writer):
+        req = Request()
 
-    async def _parse_request(self, request_reader):
-        request_line = await request_reader.readline()
-        request_line = request_line.decode()
-        method, url_string, _ = request_line.split(' ', 2)
-        method = method.upper()
-        url = parse.urlparse(url_string)
+        parser = HttpRequestParser(req)
 
-        header_lines = []
         while True:
-            l = await request_reader.readline()
-            if l == b'\r\n':
+            data = await request_reader.read(self.max_read_chunk)
+            parser.feed_data(data)
+            if req.finished:
                 break
-            header_lines.append(l.decode())
+            elif req.needs_write_continue:
+                response_writer.write(b'HTTP/1.1 100 (Continue)\r\n\r\n')
+                req.reset_state()
 
-        headers = self._parse_header_lines(header_lines)
-
-        raw_body = None
-        if method in {'POST', 'PUT'}:
-            body_parts = []
-            content_length = int(headers['Content-Length'])
-            while content_length > 0:
-                chunk_size = min(content_length, self.max_read_chunk)
-                body = await request_reader.read(chunk_size)
-                content_length -= len(body)
-                body_parts.append(body)
-            raw_body = b''.join(body_parts)
-            del body_parts
-
-        path = url.path
-        query = url.query
-
-        handler, args = self.get_handler(path)
-
-        req = Request(method, path, query, raw_body, args, headers)
+        handler, args = self.get_handler(req.path)
 
+        req.method = parser.get_method().decode().upper()
+        req.args = args
         return req, handler
 
     async def _route_request(self, handler, req, res):
@@ -101,11 +95,14 @@ class Server:
                 await handler.on_options(req, res)
             elif self.spoof_options:
                 res.headers['Allow'] = 'GET,POST,DELETE,PUT'
+            else:
+                raise HTTPError(HTTP_405)
         else:
             raise HTTPError(HTTP_405)
 
     async def _handle(self, request_reader, response_writer):
-        """Takes reader and writer from asyncio loop server and writes the response to the request.
+        """Takes reader and writer from asyncio loop server and
+        writes the response to the request.
 
         :param request_reader:
         :param response_writer:
@@ -114,7 +111,8 @@ class Server:
         res = Response()
 
         try:
-            req, handler = await self._parse_request(request_reader)
+            req, handler = await self._parse_request(
+                request_reader, response_writer)
 
             for middleware in self._middleware:
                 await middleware.process_request(req, res, handler)
@@ -144,19 +142,11 @@ class Server:
         traceback.print_exc()
 
     def _write_response(self, res, writer):
-        writer.write(b'HTTP/1.0 %s\r\n' % res.status_code.encode())
+        writer.write(b'HTTP/1.1 %s\r\n' % res.status_code.encode())
         for key, value in res.headers.items():
             writer.write(key.encode() + b': ' + str(value).encode() + b'\r\n')
         for key, value in res.cookies.items():
-            if isinstance(value, tuple):
-                value, duration = value
-                if isinstance(duration, datetime):
-                    format = duration.strftime('%a %d %b %Y %H:%M:%S GMT').encode()
-                    writer.write(b'Set-Cookie: %s=%s;expires=%s\r\n' % (key.encode(), str(value).encode(), format))
-                elif isinstance(duration, int):
-                    writer.write(b'Set-Cookie: %s=%s;max-age=%d\r\n' % (key.encode(), str(value).encode(), duration))
-            else:
-                writer.write(b'Set-Cookie: %s=%s\r\n' % (key.encode(), str(value).encode()))
+            write_cookie(writer, key, value)
         writer.write(b'\r\n')
         for chunk in res._chunks:
             writer.write(chunk)
